<CFSETTING ENABLECFOUTPUTONLY="YES">
<!--- || BEGIN FUSEDOC ||
	
|| PROPERTIES ||
	Application: cfmail replacement
	Module Name: cf_mail
	File Name: mail
	Version: 1.0
	Author: Maxim Paperno <MPaperno@WorldDesign.com>
	Copyright: 2003 World Design Group, Inc.
				http://www.worldDesign.com/
	
|| RESPONSIBILITIES ||
	Drop-in replacement for CFMAIL tag which utilizes iMSMail COM object to do the actual mail sending.  This reliably places properly-formatted mail and control files in the inFusion Mail Server outgoing queue.  Significantly more reliable than cfmail.

|| HISTORY ||
	05.28.03 : created
	mm.dd.yy : updated (by: )
	
|| ATTRIBUTES ||
	(all are strings in attributes scope unless specified otherwise; '+' means required;)
	
	cfmail-compatible attributes (see cfmail docs for descriptions):
		+> to
		+> from
		+> subject
		-> cc
		-> bcc
		-> type
		-> mimeAttach
		-> mailerID : default is "ColdFusion Script"
		-> query : queries are not supported so if this has a value, an error is returned
		
		note that the following cfmail attributes are NOT supported:
			maxrows,startrow,group,groupcasesensitive,server,port,timeout
		
	cf_mail specific attributes:
		-> debug : boolean : turns on cf_mail debug output showing various internal variable values; 
				default: false
		-> throwerror : boolean : if set to false then errors are not thrown but instead returned in status vars;
				default: true (same behavior as cfmail)
		-> wordwrap : int : column to wrap mail body text at, zero to disable; default: 0 (disabled)
		-> log : boolean :  [ currently has no effect ] turn on logging of the mail being sent; default: true 

	return values:
		<+ caller.cf_mail.statusCode : int : Numerical result code (see below for list)
		<+ caller.cf_mail.statusMessage : string : Explanatory result string
	
|| USAGE ||
	Use as a drop-in replacement for CFMAIL.  Does not support the QUERY feature, but everything else is identical to cfmail.
	
	Example:
	<cf_mail to="somewhere@example.com" 
			 bcc="another@example.com" 
			 from="someone@example.com" 
			 subject="Test Subject">
		<cf_mailparam name="X-Mailier" value="foobar">
		<cf_mailparam file="c:\temp\test.txt">
		... message text ...
	</cf_mail>
	

|| RETURN CODES ||
	0-99:Success; 100-199:Warning; 200-299:Error
	
	0 : Default success result
	
	200 : Unknown error condition (eg. error creating object)
	210 : No Closing Tag
	211 : Non-blank QUERY attribute detected (tag does not support it)
	215 : TO is blank or not an email address
	216 : FROM is blank or not an email address
	220 : SMTPTO is blank or not an email address (after processing TO address(es))
	221 : SMTPFROM is blank or not an email address (after processing FROM address)

|| END FUSEDOC || --->

<!--- ####   ATTRIBUTE DECLARATIONS   #### --->

<!--- required attributes --->
<cfparam name="attributes.to" type="string">
<cfparam name="attributes.from" type="string">
<cfparam name="attributes.subject" type="string">

<!--- optional attributes --->
<cfparam name="attributes.cc" default="" type="string">
<cfparam name="attributes.bcc" default="" type="string">
<cfparam name="attributes.type" default="" type="string">
<cfparam name="attributes.mimeAttach" default="" type="string">
<cfparam name="attributes.mailerID" default="ColdFusion Script" type="string">
<cfparam name="attributes.query" default="" type="string">


<!--- cf_mail specific attributes --->

  <!--- prints debug info --->
<cfparam name="attributes.debug" default="0" type="boolean"> 
  <!--- determines if errors terminate execution (default) or fail "softly" --->
<cfparam name="attributes.throwError" default="1" type="boolean"> 
  <!--- column to wrap text at, zero to disable (passed to imsMail) --->
<cfparam name="attributes.wordwrap" default="0" type="numeric"> 
  <!--- log the mail sending (passed to imsMail) --->
<cfparam name="attributes.log" default="1" type="boolean"> 

<!--- Protect against no mailparam sub-tags --->
<cfparam Name="thisTag.mailparamAttribs" default="#arrayNew(1)#">


<!--- ####   VARIABLE DECLARATIONS   #### --->

<!--- file attachments --->
<cfset attachments = attributes.mimeAttach>

<!--- structure of parameters to pass to iMSMail (cfx or object)
	( structMailParams.smtpTo='to@dddress', structMailParams.smtpFrom='from@dddress', etc. ) --->
<cfset structMailParams = StructNew()>

<!--- mail headers array
	( aryHeadersStruct[i].name='header-name', aryHeadersStruct[i].value='header value' ) --->
<cfset aryHeadersStruct = ArrayNew(1)>


<!--- ####   UTILITY FUNCTIONS   #### --->

<cfif thisTag.ExecutionMode IS "end">
	<!--- have to include the custom functions because if they're imbedded CF throws an error. grrrr --->
	<cfinclude template="inc_mail_util.cfm">
</cfif>


<!--- ####   START PROCESSING   #### --->

<cfif thisTag.ExecutionMode IS "start">

	<!--- make sure there is an end tag --->
	<cfif NOT thisTag.HasEndTag>
		<cfthrow type="Application" 
			errorcode="210" 
			message="CF_MAIL: This tag must have a matching closing tag (cf_mail..../cf_mail)">
	</cfif>
	
	<!--- this tag cannot support the query attribute so return an error if it's being used --->
	<cfif Len(attributes.query)>
		<cfthrow type="Application" 
			errorcode="211" 
			message="CF_MAIL: This tag does not support the QUERY attribute. Manually loop over this tag to achieve the same result.">
	</cfif>
	
 <cfelse>   <!--- everything else can happen in end exec mode --->

	<cftry>
	
		<!--- error checking --->
		
		<cfif NOT Len(Trim(attributes.to)) OR NOT Find("@", attributes.to)>
			<cfthrow type="Application" 
				errorcode="215" 
				message="CF_MAIL: TO attrbute cannot be blank or is not an email address">
		</cfif>
		<cfif NOT Len(Trim(attributes.from)) OR NOT Find("@", attributes.from)>
			<cfthrow type="Application" 
				errorcode="216" 
				message="CF_MAIL: FROM attrbute cannot be blank or is not an email address">
		</cfif>
		
		<!--- end error check --->
	
		<cfscript>
		
		// format date and time string for use later on in Date header
		timezone = GetTimezoneInfo();
		dateTimeString = DateFormat(Now(), "ddd, d mmm yyyy ") & 
							TimeFormat(Now(), "HH:mm:ss ") & 
							IIf( Sgn(timezone.utcHourOffset) EQ 1, DE("-"), DE("+") ) & 
							NumberFormat(timezone.utcHourOffset, "00") & "00";
		
		// assemble all addresses to send to including cc and bcc
		allAddressees = attributes.to;
		if ( Len(Trim(attributes.cc)) )
			allAddressees = ListAppend(variables.allAddressees, attributes.cc);
		if ( Len(Trim(attributes.bcc)) )
			allAddressees = ListAppend(variables.allAddressees, attributes.bcc);
		
		// start setting parameters for the final mailer process
		// cleanAddrList() is a custom function included in inc_mail_util.cfm
		structMailParams.smtpTo = cleanAddrList(variables.allAddressees);
		structMailParams.smtpFrom = ListFirst(cleanAddrList(attributes.from));
		if ( attributes.type IS "HTML" )
			structMailParams.html = thisTag.GeneratedContent;
		 else
			structMailParams.body = thisTag.GeneratedContent;
		
		structMailParams.log = attributes.log;
		structMailParams.wordWrap = attributes.wordwrap;
		structMailParams.failTo = structMailParams.smtpFrom;
		structMailParams.warnTo = structMailParams.smtpFrom;
	
		// now add some common headers
		// addHeader() is a custom function included in inc_mail_util.cfm
		addHeader("Subject", attributes.subject);
		addHeader("To", attributes.to);
		addHeader("From", attributes.from);
		addHeader("Date",  variables.dateTimeString);
		addHeader("X-Mailer", attributes.mailerID);
		
		if ( Len(Trim(attributes.cc)) ) {
			addHeader("CC", attributes.cc);
		}
	
		// loop over cf_mailparam tag-specified attributes (if any) and
		// add in attachments and/or custom headers specified therein
		for (i=1; i LTE ArrayLen(thisTag.mailparamAttribs); i=i+1) {
			if ( Len(Trim(thisTag.mailparamAttribs[i].file)) ) {
				// buld a csv list of attachment files 
				attachments = ListAppend(variables.attachments, Trim(thisTag.mailparamAttribs[i].file));
			} else if ( Len(Trim(thisTag.mailparamAttribs[i].name)) ) {
			 	// add headers to the header array
				addHeader( Trim(thisTag.mailparamAttribs[i].name), Trim(thisTag.mailparamAttribs[i].value) );
			}
		}
		
		</cfscript>
	
		<!--- final syntax check --->
		
		<cfif NOT Len(Trim(structMailParams.smtpTo))>
			<cfthrow type="Application" 
				errorcode="220" 
				message="CF_MAIL: The SMTPTO attbute ended up blank. Please check your TO address syntax.">
		</cfif>
		<cfif NOT Len(Trim(structMailParams.smtpFrom))>
			<cfthrow type="Application" 
				errorcode="221" 
				message="CF_MAIL: The SMTPFROM attbute ended up blank. Please check your FROM address syntax.">
		</cfif>
		
		<!--- end syntax check --->
	
	
		<!--- ####   Here's where we start the sending process   ### --->
		
		<!--- create the imsmail object --->
		<cfobject type="COM" name="imsMail" class="iMSMail.SendMail" action="CREATE">

		<cfscript>
		
		// set all the parameters
		for (i IN variables.structMailParams) {
			"imsMail.#i#" = variables.structMailParams[i];
		}
		// set all headers
		for (i=1; i LTE ArrayLen(variables.aryHeadersStruct); i=i+1) {
			imsMail.setHeader(variables.aryHeadersStruct[i].name, variables.aryHeadersStruct[i].value);
		}
		// set any attachments specified
		for (i=1; i LTE ListLen(variables.attachments); i=i+1) {
			imsMail.addAttachment(ListGetAt(variables.attachments, i), "");
		}
		
		// send the mail!
		imsMail.sendmail();
		
		</cfscript>
		
		<!--- if object returned an error (non zero result), throw it --->
		<cfif imsMail.result>
			<cfthrow type="Application" 
				errorcode="#imsMail.result#" 
				message="CF_MAIL: IMSCOM: #imsMail.resultString#">
		</cfif>
	
		<cfset caller.cf_mail.statusCode = 0>
		<cfset caller.cf_mail.statusMessage = "CF_MAIL: Send mail successful">
	
		<!--- DEBUG INFO --->
		<cfif attributes.debug>
			<cfoutput>
				<b>cf_mail attributes:</b><br>
				<cfdump var="#attributes#">
				<b>thisTag structure:</b><br>
				<cfdump var="#thisTag#">
				<b>structMailParams:</b><br>
				<cfdump var="#variables.structMailParams#">
				<b>headers array:</b><br>
				<cfdump var="#variables.aryHeadersStruct#">
				<b>attachments list:</b><br>
				<cfdump var="#variables.attachments#"><br>
			</cfoutput>
		</cfif>
	
		<cfcatch type="Any">
			<cfif attributes.throwError>
				<!--- just rethrow the error --->
				<cfrethrow>
			 <cfelse>
			 	<!--- return the error in the result vars --->
				<cfset caller.cf_mail.statusCode = IIf( Len(cfcatch.errorCode), "cfcatch.errorCode", 200 )>
				<cfset caller.cf_mail.statusMessage = cfcatch.message & "<br>" & 
											IIf( Len(cfcatch.detail), "cfcatch.detail", DE("") )>
			</cfif>
		</cfcatch>
	
	</cftry>

</cfif>

<cfset thisTag.GeneratedContent = "">

<CFSETTING ENABLECFOUTPUTONLY="NO">